"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = configure;

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

var _vm = _interopRequireDefault(require("vm"));

var _crypto = require("crypto");

var _resolve = _interopRequireDefault(require("resolve"));

var _chokidar = _interopRequireDefault(require("chokidar"));

var _core = require("@babel/core");

var _mkdirp = require("mkdirp");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var called = false;

function configure(entrypoint, overrideRequires, opts, onValue, onError) {
  if (called) throw new Error('You can only use babel-live once per project');
  called = true;
  opts = opts || {};
  var babelCache = opts.babelCache ? _path["default"].resolve(opts.babelCache) : null;

  if (babelCache) {
    delete opts.babelCache;
    (0, _mkdirp.sync)(babelCache);
  }

  var resolveOptions = opts.resolveOptions || {};

  if (resolveOptions) {
    delete opts.resolveOptions;
  }

  var fallbackRequire = opts.fallbackRequire;

  if (fallbackRequire) {
    delete opts.fallbackRequire;
  }

  function handleError(ex) {
    var err = '';

    if (ex.codeFrame) {
      err = ex.message + '\n' + ex.codeFrame;
    } else {
      err = ex.stack;
    }

    if (onError) {
      onError(err);
    } else if (fallbackRequire) {
      console.error(err);
    } else {
      throw err;
    }

    if (fallbackRequire) {
      var result = babelRequire(fallbackRequire);
      onValue(result);
      return result;
    }
  }

  var requireInProgress = false;
  var extraRequireNeeded = false;
  var requireCache = {}; // filename => fn(module, exports, require, __filename, __dirname)

  var moduleCache = {};
  var maps = {};
  /*
  // TODO: source maps in a VM are different
  sourceMapSupport.install({
    handleUncaughtExceptions: false,
    environment: 'node',
    retrieveSourceMap(source) {
      const map = maps[source];
      if (map) {
        return {
          url: null,
          map,
        };
      } else {
        return null;
      }
    },
  });
  */

  function invalidate(filename) {
    console.log('detected file change: ' + filename);
    requireCache = {};
    moduleCache[filename] = null;
    doRequire();
  }

  function doRequire() {
    if (!requireInProgress) {
      requireInProgress = true;

      try {
        var result = babelRequire(entrypoint);
        onValue(result);
      } catch (ex) {
        handleError(ex);
      }

      setTimeout(function () {
        requireInProgress = false;

        if (extraRequireNeeded) {
          extraRequireNeeded = false;
          doRequire();
        }
      }, 2000);
    } else {
      extraRequireNeeded = true;
    }
  }

  var watching = {};

  function watch(filename) {
    if (watching[filename]) return;
    watching[filename] = true;

    var w = _chokidar["default"].watch(filename, {
      persistent: true,
      usePolling: true,
      interval: 100
    });

    w.on('error', function (err) {
      console.log('error watching file');
      console.log('probably nothing to worry about');
      console.log(err.message);
    });
    w.on('change', function () {
      return invalidate(filename);
    });
  }

  function babelRequire(filename) {
    filename = _path["default"].resolve(filename);

    if (!/\.js$/.test(filename) && !(resolveOptions && resolveOptions.extensions && resolveOptions.extensions.some(function (ext) {
      return filename.endsWith(ext);
    }))) {
      return require(filename);
    }

    if (!/node_modules/.test(filename)) {
      watch(filename);
    }

    if (requireCache[filename]) return requireCache[filename].exports;
    var fn = moduleCache[filename];

    if (!fn) {
      var src = !/node_modules/.test(filename) ? babelLoad(filename) : _fs["default"].readFileSync(filename, 'utf8');
      fn = _vm["default"].runInThisContext('(function(module,exports,require,__filename,__dirname){' + src + '\n})', filename);
      moduleCache[filename] = fn;
    }

    requireCache[filename] = {
      exports: {}
    };

    function proxiedRequire(id) {
      if (overrideRequires[id]) {
        return overrideRequires[id];
      }

      if (_resolve["default"].isCore(id)) {
        return require(id);
      }

      var p = _resolve["default"].sync(id, _objectSpread({
        basedir: _path["default"].dirname(filename)
      }, resolveOptions, {
        extensions: [].concat(_toConsumableArray(resolveOptions.extensions || []), ['.js', '.json', '.node'])
      }));

      return babelRequire(p);
    }

    proxiedRequire.resolve = function (id) {
      return _resolve["default"].sync(id, {
        basedir: _path["default"].dirname(filename),
        extensions: ['.js', '.json', '.node']
      });
    };

    var sandbox = {
      'module': requireCache[filename],
      'exports': requireCache[filename].exports,
      'require': proxiedRequire,
      '__filename': filename,
      '__dirname': _path["default"].dirname(filename)
    };
    fn(sandbox.module, sandbox.exports, sandbox.require, sandbox.__filename, sandbox.__dirname);
    return requireCache[filename].exports;
  }

  function babelLoad(filename) {
    var src = _fs["default"].readFileSync(filename, 'utf8');

    opts.filename = filename;
    var hash, result;

    if (babelCache) {
      hash = (0, _crypto.createHash)('sha1').update(src).digest('hex');

      try {
        result = JSON.parse(_fs["default"].readFileSync(_path["default"].join(babelCache, hash + '.json'), 'utf8'));
      } catch (ex) {
        if (ex.code !== 'ENOENT') {
          throw ex;
        }
      }
    }

    if (!result) {
      result = (0, _core.transform)(src, _objectSpread({}, opts, {
        sourceMaps: 'both',
        ast: false
      }));

      if (babelCache) {
        _fs["default"].writeFileSync(_path["default"].join(babelCache, hash + '.json'), JSON.stringify(result));
      }
    }

    maps[filename] = result.map;
    return result.code;
  }

  try {
    var result = babelRequire(entrypoint);
    onValue(result);
    return {
      reRun: function reRun() {
        requireCache = {};
        doRequire();
      }
    };
  } catch (ex) {
    return handleError(ex);
  }
}